#!/usr/bin/perl
# schwartzian_benchmark.pl
use v5.10;
use strict;
use Benchmark;
use Cwd qw(cwd);

use Getopt::Std;
getopts( 'f:g:i:s:', \ my %opts );

my $files      = $opts{f} // 100;
my $iterations = $opts{i} // 10_000;
my $seconds    = 0 - abs( $opts{'s'} // -2 );

use File::Temp qw(tempdir);
my $dir = tempdir( CLEANUP => 1 );

my( $original_cwd ) = cwd();
chdir( $dir ) or die "Could not change to $dir: $!";
foreach ( 1 .. $files ) {
    open my($fh), '>', "$0.$_.tmp" or die "Could not create a file: $!";
    print { $fh } time();
    }

$L::glob = $opts{g} // '*';
@L::files = glob $L::glob;

print "Testing with " . @L::files . " files\n";

my $transform = q|map $_->[0], sort { $a->[1] <=> $b->[1] } map [ $_, -M ]|;
my $sort      = q|sort { -M $a <=> -M $b }|;

my $code = {
    assign               =>  q| my @r = @L::files |,
    'glob'               =>  q| my @r = glob $L::glob |,
    sort_names           =>  q| sort { $a cmp $b } @L::files |,
    sort_names_assign    =>  q| my @r = sort { $a cmp $b } @L::files |,
    sort_times_assign    => qq| my \@r = $sort \@L::files |,

    ordinary_orig        => qq| my \@r = $sort glob \$L::glob |,
    ordinary_mod         => qq| my \@r = $sort \@L::files |,
    schwartz_orig        => qq| $transform, glob \$L::glob |,
    schwartz_orig_assign => qq| my \@r = $transform, glob \$L::glob |,
    schwartz_mod         => qq| my \@r = $transform, \@L::files |,
};

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
printf "Timing for %d CPU seconds...\n", $seconds ;
timethese( $seconds, $code );

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
print "\n", "-" x 73, "\n\n";
print "Timing for $iterations iterations\n";

timethese( $iterations, $code );
chdir( $original_cwd );